/*
 *****************************************************************************
 * audio_rc.c
 Title: audio signal acquisition from radio control transmitter
 Authors:
  Joel Lienard
  Lionel Cailler
  greg
 
 Purpose:  Audio signal acquisition from radio control transmitter.
  This code is based on origianl  Joel Lienard source code modified to support a
  portable library to a wide list of operating systems : http://www.portaudio.com

 It's free, it works, enjoy.  This  is an open source project, so if you  don't
 like the way something works, help us fix it!


 * CHANGE HISTORY:
   03nov2003 add recompile with port_audio_v18_1 and text display improvements.
   30dec2003 correction made in audio_rc.c by greg : correction made in audio_rc.c 
    1)   while testing audio_rc.c independently of crrcsim, i saw that it generated Pa_Audio errors.
        this was corrected by moving the call to Pa_Initialize() before the call to the first Pa_Audio
        function, Pa_GetDefaultInputDeviceID().  however, the interface was still returnign zero values.
    3)  i then added a call to Pa_Sleep(1) before the loop calling get_data() in
        get_data_from_audio_interface().  now the audio interface was active and responding well.

 *
 *****************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include "portaudio.h"
#include "audio_rc.h"

#include <stdio.h>
#include <stdlib.h>

#define SAMPLE_RATE  (44100)
#define NUM_SAMPLES   (2000)
#define NUM_CHANNELS    (2)

typedef float SAMPLE;

typedef struct
{
    int         frameIndex;  /* Index into sample array. */
    int         frameSize;
    int         SamplesCounter;
    int 	LastSamplesCounter;
    SAMPLE      *recordedSamples;
}
paAudioData;

paAudioData AudioData;
PortAudioStream *stream;
int inputDevice;


// --------------------------------------------------------------------
/* This routine will be called by the PortAudio engine when audio is needed.
** It may be called at interrupt level on some machines so don't do anything
** that could mess up the system like calling malloc() or free().
*/

/* purpose of this routine is to update a rotating buffer
 *  index of buffer is AudioData.frameIndex
 *  buffer size is AudioData.frameSize
 *  number of samples stored since audio_rc_open() called
 */
static int recordCallback( void *inputBuffer, void *outputBuffer,
                           unsigned long framesPerBuffer,
                           PaTimestamp outTime, void *userData )
{
    paAudioData *data = (paAudioData*)userData;
    SAMPLE *rptr = (float*)inputBuffer;
    SAMPLE *wptr;
    unsigned long i;
    unsigned long framesLeft = data->frameSize - data->frameIndex;

    (void) outputBuffer; /* Prevent unused variable warnings. */
    (void) outTime;

    data->SamplesCounter+=framesPerBuffer;
    
    wptr = &data->recordedSamples[data->frameIndex];
    if( framesPerBuffer <= framesLeft )
        for( i=0; i<framesPerBuffer; i++ )
            { *wptr++=*rptr++; rptr++; }  // assume NUM_CHANNELS = 2
    else
    {
        for( i=0; i<framesLeft; i++ )
            { *wptr++=*rptr++; rptr++; } // assume NUM_CHANNELS = 2

        wptr = &data->recordedSamples[0];
        for( i=0; i<framesPerBuffer-framesLeft; i++ )
            { *wptr++=*rptr++; rptr++; }  // assume NUM_CHANNELS = 2
    }

    return 0;
}


// --------------------------------------------------------------------
static int get_data(float *values,int *nvalues)
       {
       float x, px, max, min, moy;
        int imp;
        int nval = AudioData.frameSize;
        int i, chanel=0;
        float time, dt;
        float *sig=AudioData.recordedSamples;

                max= -100000;
                min=  100000;
                for(i=0;i<nval;i++){
                        x = sig[i];
                        if (x>max) max=x;
                        if (x<min) min=x;
                        }
                moy = (max+min)/2;
                x=0;
                chanel=-1; time = 0;
                for(i=0;i<nval;i++){
                        time++;
                        if (time >100){
                                if (chanel < 0) chanel=0; //start
                                if(chanel>0) break;//end
                                }
                        px=x;
                        x = sig[i];
                        if((x>moy) && (px<moy) ) imp = 1; else imp = 0;
                        if(imp){
                                dt = (moy-px)/(x-px);
                                if (chanel>=0){
                                        if(chanel>0) {
                                                values[chanel-1]=(time+dt)/44.1-1.5;
                                                }
                                        chanel ++;
                                        if(chanel>10) break;
                                        }
                                time = -dt;
                                }
                        }
        *nvalues=chanel-1;
        return 1;
        }
        
// --------------------------------------------------------------------
int get_data_from_audio_interface(float *values){
        static float vals[11];
        static int nvals=0;
        int i,n;

       /*
        * there is no need to update values more than 50 times a second
        * then lest check here
        */
        
		Pa_Sleep (1);

        n=AudioData.SamplesCounter;
        if (n-AudioData.LastSamplesCounter>AudioData.frameSize){
            AudioData.LastSamplesCounter = n;                         // lets update values
            if(get_data(vals,&n)){
                for(i=0;i<11;i++)values[i]=vals[i];
                nvals = n;
                }
        }else
       /*
        * LastSamplesCounter & SamplesCounter saturation management
        */
        if (n<AudioData.LastSamplesCounter) AudioData.LastSamplesCounter = n;
        
        
        return nvals;
        }

// --------------------------------------------------------------------
int audio_rc_open(){
    int i;
    PaError    err;

    err = Pa_Initialize();
    if( err != paNoError ) goto error;

    inputDevice = Pa_GetDefaultInputDeviceID();
    AudioData.frameSize = NUM_SAMPLES; /* Record for a few samples. */
    AudioData.frameIndex = 0;
    AudioData.LastSamplesCounter=0;
    AudioData.SamplesCounter=0;

    AudioData.recordedSamples = (SAMPLE *) malloc( AudioData.frameSize * sizeof(SAMPLE) );
    if( AudioData.recordedSamples == NULL )
    {
        printf("Could not allocate record array.\n");
        exit(1);
    }
    for( i=0; i<AudioData.frameSize; i++ ) AudioData.recordedSamples[i] = 0;


    /* Record some audio. -------------------------------------------- */
    err = Pa_OpenStream(
              &stream,
              inputDevice,
              NUM_CHANNELS,               /* NUM_CHANNELS : stereo input */
              paFloat32,
              NULL,
              paNoDevice,
              0,
              paFloat32,
              NULL,
              SAMPLE_RATE,
              1024,            /* frames per buffer */
              0,               /* number of buffers, if zero then use default minimum */
              0, //paDitherOff,    /* flags */
              recordCallback,
              &AudioData );
    if( err != paNoError ) goto error;

    err = Pa_StartStream( stream );
    if( err != paNoError ) goto error;

    return 0;
    
error:
    Pa_Terminate();
    fprintf( stderr, "An error occured while using the portaudio stream\n" );
    fprintf( stderr, "Error number: %d\n", err );
    fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );

    return -1;
}
        
// --------------------------------------------------------------------
int audio_rc_close(){
    PaError    err;

    err = Pa_CloseStream( stream );
    if( err != paNoError ) goto error;
    free( AudioData.recordedSamples );

    Pa_Terminate();
    return 0;
    
error:
    Pa_Terminate();
    fprintf( stderr, "An error occured while using the portaudio stream\n" );
    fprintf( stderr, "Error number: %d\n", err );
    fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );

    return -1;
}
        
