/***************************************************************************

 Title:		crrc_checkopts.c

 DESIGNED:	Kees Lemmens, July 2004
 CODED:   	KL
 MAINTAINER:	KL
 LAST MODIFIED:	July 2004
----------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>

extern char   *optarg;
extern int    optind;

#include "crrc_main.h"

void crrc_usage(char *progname)
{
   fprintf(stderr,"\nUsage  : %s [options] [<plane>]\n",progname);
   fprintf(stderr,"\nNote that these options override default settings in crrcsim.cfg !\n");
   fprintf(stderr,  "Options: -l <string>   : location Davis, Medfield or Cape_Cod\n");
   fprintf(stderr,  "         -f            : use fullscreen\n");
   fprintf(stderr,  "         -v            : verbosity true\n");
   fprintf(stderr,  "         -i <string>   : input method : Mouse, Joystick or Keyboard\n");
   fprintf(stderr,  "         -j <string>   : joystick 2 motion: Aileron or Rudder\n");
   fprintf(stderr,  "         -m <string>   : mouse x motion : Aileron or Rudder\n");
   fprintf(stderr,  "         -w <value>    : wind velocity in ft/sec\n");
   fprintf(stderr,  "         -d <value>    : wind direction in deg (0-360)\n");
   fprintf(stderr,  "         -c <value>    : color_depth in bits per pixel\n");
   fprintf(stderr,  "         -x <value>    : x_resolution in pixels\n");
   fprintf(stderr,  "         -y <value>    : y_resolution in pixels\n");
   fprintf(stderr, "\n");
}

int crrc_checkopts(int argc,char *argv[])	/* check and set options flags */
{
   int c;
   int opt_err = 0;
   
   while ((c = getopt(argc, argv, "l:fvi:j:m:w:d:c:x:y:h")) != EOF)
   {
      switch (c)
      {
       case 'l': /* airport location */
	 if      (strcasecmp(optarg,"DAVIS")==0)
	   location=DAVIS;
	 else if (strcasecmp(optarg,"MEDFIELD")==0)
	   location=MEDFIELD;
	 else if (strcasecmp(optarg,"CAPE_COD")==0)
	   location=CAPE_COD;
	 break;
       case 'f':
	   fullscreen=TRUE;
	 break;
       case 'v':
	   verbose=TRUE;
	 break;
       case 'i':
	 if      (strcasecmp(optarg,"MOUSE")==0)
	    input_method=MOUSE;
	 else if (strcasecmp(optarg,"KEYBOARD")==0)
	    input_method=KEYBOARD;
	 else if (strcasecmp(optarg,"JOYSTICK")==0)
	    input_method=JOYSTICK;
	 break;
       case 'j':
	 if      (strcasecmp(optarg,"AILERON")==0)
	   joystick_bind_x[2] = AILERON;
	 else if (strcasecmp(optarg,"RUDDER")==0)
	   joystick_bind_x[2] = RUDDER;
	 break;
       case 'm':
	 if      (strcasecmp(optarg,"AILERON")==0)
	   mouse_bind_x = AILERON;
	 else if (strcasecmp(optarg,"RUDDER")==0)
	   mouse_bind_x = RUDDER;
	 break;
       case 'w':
	 wind_velocity=(float)atof(optarg);
	 break;
       case 'd':
	 wind_direction=(float)atof(optarg);
	 break;
       case 'c':
	 color_depth=atoi(optarg);
	 break;
       case 'x':
	 x_resolution=atoi(optarg);
	 break;
       case 'y':
	 y_resolution=atoi(optarg);
	 break;
       case 'h':
	 crrc_usage(argv[0]);
	 exit(0);
	 break;
       default:
	 opt_err = 1;
      }
      
      if (opt_err)
      {
	 crrc_usage(argv[0]);
	 return 1;
      }
   }
   return 0;
}

