/*****************************************************************************
Title: CRRCsim, the Charles River Radio Control Club Flight Simulator Project
Authors:
Jan Kansky:  Programming
Mark Drela:  Aerodynamics

Purpose:  The idea for CRRCsim is to take away any last excuse you might
have for not using a flight simulator to keep your thumbs certified for RC
flying.   It's free, it works, enjoy.  This is an open source project, so if
you don't like the way something works, help us fix it!

Thanks:
  Bruce Jackson for the LaRCsim framework.
  Flight Gear project for the sky sphere.

Contacts:
If you'd like to help with CRRCSIM, then send me an email!
    email                : kansky@ll.mit.edu
*****************************************************************************/

#ifndef CRRC_MAIN_H
#define CRRC_MAIN_H

#include <SDL.h>
#ifdef WIN32
 #include <windows.h>
 #include "gl.h"
 #include "glu.h"
 #include "glut.h"
 #include "tube.h"
 #include "dlportio.h"
#endif
#if defined(__APPLE__) || defined(MACOSX)
 #include <GLUT/glut.h>
// #include <GLUT/gutil.h>
 #include <GLUT/tube.h>
#endif  // __APPLE__
#ifdef linux
 #include <GL/glut.h>
// #include <GL/gutil.h>
 #include <GL/tube.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#if defined(__APPLE__) || defined(MACOSX)
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <sys/time.h>
// #include <asm/io.h>
#endif  // __APPLE__
#ifdef linux
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <sys/time.h>
 #include <asm/io.h>
#endif
#include <fcntl.h>
#include <math.h>
#include "ls_generic.h"
#include "ls_interface.h"
#include "ls_constants.h"
#ifdef linux
  #include "mbuff.h"
#endif

#ifndef TRUE
 #define TRUE 1
#endif

#ifndef FALSE
 #define FALSE 0
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

// 1/exp(exp(1)), needed for mix_exp
#ifndef M_1_E_E
#define M_1_E_E 0.065988
#endif

// exp(1), needed for mix_exp
#ifndef M_E
#define M_E 2.7182818284590452354
#endif

/*****************************************************************************/
#define DAVIS             1    // CRRC Davis flying field in Sudbry, Mass
#define MEDFIELD          2    // CRRC Medfield flying site in Medfield, Mass
#define CAPE_COD          3    // Slope Soaring at Cape Cod, Massachusetts 

#define AILERON           1    // some things you can bind to
#define ELEVATOR          2    // joystick axes and other axes
#define RUDDER            3    // 0 = NOTHING
#define THROTTLE          4

#define NOTHING           0    // some things you can bind to
#define RESUME            1    // joystick buttons and other keys or buttons
#define PAUSE             2 
#define RESET             3
#define INCTHROTTLE       4
#define DECTHROTTLE       5

#define FUTABA				0
#define AIRTRONICS			1
#define HITEC				2
#define JR					3
#define KEYBOARD          1    // Use keyboard for fligt controls
#define MOUSE             2    // Use mouse for flight controls
#define JOYSTICK          3    // Use joystick for inputs
#define RCTRAN            6    // Use LPT interrupt under Linux for inputs
#define SERIAL            7    // Use Breed Serial port interface
#define PARALLEL1         8    // Use Kansky parallel port interface
#define PARALLEL2         9    // Use Kansky parallel port interface
#define PARALLEL3         10   // Use Kansky parallel port interface


#define MOUSE_FOV         1
#define ARROW_KEY_FOV     2

#define SMOOTH  1
#define FLAT    2
#define FOG_R 1.0f
#define FOG_G 1.0f
#define FOG_B 1.0f
#define FOG_A 1.0f
#define occupancy_grid_size 128
#define occupancy_grid_res 100
#define area_of_influence 5
#define turb_grid_size 1024   // 1 cm resolution, 10.24m max size.

/*****************************************************************************/
// Typedef section :

typedef struct
{
   float center_x_position; // Center position of thermal on ground
   float center_y_position; // Center position of thermal on ground
   float radius;            // Radius of thermal column ft 
   float boundary_thickness; // 1/e width of transition into thermal core
   float strength;          // Vertical component strength in ft/s
   float lifetime;          // How long should the thermal last in sec
   float time_expired;      // How long has the thermal been around in sec
   int xcoord;              // X coordinate in thermal occupancy grid
   int ycoord;              // Y coordinate in thermal occupancy grid
   void *next_thermal;
} thermal;

typedef struct
{
   char *extrusion_name;
   gleDouble up[3];
   int num_points_on_path;
   GLfloat translate_x,translate_y,translate_z;
   GLfloat rotate_x,rotate_y,rotate_z;
   gleDouble **point_array;
   gleAffine *scaling;
   GLfloat **colors;
   int num_contour_elements;
   gleDouble **contour;
   gleDouble **contour_normal_vectors;
   void *next_extrusion;
} extrusion;

typedef struct
{
   char *cylinder_name;
   GLfloat translate_x,translate_y,translate_z;
   GLfloat rotate_x,rotate_y,rotate_z;
   GLfloat radius_1,radius_2,length;
   GLfloat radial_sampling,longitudinal_sampling;
   GLfloat colors[4];
   void *next_cylinder;
} cylinder;

typedef struct
{
   char *sphere_name;
   GLfloat translate_x,translate_y,translate_z;
   GLfloat rotate_x,rotate_y,rotate_z;
   GLfloat radius;
   GLfloat slices,stacks;
   GLfloat colors[4];
   void *next_sphere;
} sphere;

typedef struct
{
   char *triangle_name;
   unsigned long int num_triangles;
   GLfloat translate_x,translate_y,translate_z;
   GLfloat rotate_x,rotate_y,rotate_z;
   GLfloat colors[4];
   GLfloat **vertices;
   GLfloat **normals;
   void *next_triangle;
} triangle;

/*****************************************************************************/
// Functions defined outside crrc_main.c and used inside crrc_main.c :
int crrc_checkopts(int argc,char *argv[]);
void key_down(SDL_keysym *keysym);
int read_config_file(char *filename);

// Functions define inside crrc_main.c and used in- or outside crrc_main.c :
int read_airplane_data(char *datafile);
void read_airplane_file(int opt,char *plane);
void initialize_graphics(void);
void initialize_flight_model(void);
void initialize_window(void);
void fgSkyVerticesInit(void);
void draw_aky(void);
void draw_medfield_field(void);
void draw_davis_field(void);
void draw_airplane(void);
void display(void);
void calculate_wind(double X_cg,double Y_cg,double H_cg,double *Vel_north,
		    double *Vel_east, double *Vel_down);
void reshape(int w, int h);
void mouse_button(SDL_MouseButtonEvent *);
void mouse_motion(SDL_MouseMotionEvent *);
void joystick_motion(SDL_JoyAxisEvent *);
void joystick_button(SDL_JoyButtonEvent *);
void idle(void);
void init_parallel_interface(int);

void resume_sim();
void pause_sim();
void reset_sim();

/*****************************************************************************/
//Variables used outside crrc_main.c :

// Configuration :
extern int verbose;
extern int location;      // Davis, Medfield or Cape Cod will be the options
extern int fullscreen;    // Indicates whether to run in full screen mode
extern int input_method;  // 1=keyboard 2=mouse 3=joystick 6=rctran 7=Serial Port
extern float scale_aileron; // 1.0=no invert -1.0=invert => use as factor.
extern float scale_elevator;
extern float scale_rudder;
extern float scale_throttle;
extern float trim_aileron; // trim offset, 0.0 = neutral, -0.5 .. 0.5
extern float trim_elevator;
extern float trim_rudder;
extern float trim_throttle;
extern float nrate_aileron; // normal rate, scales output from 0 .. 100%
extern float nrate_elevator;
extern float nrate_rudder;
extern float nrate_throttle; 
extern float exp_aileron; // mix exp, 0.0 = no exp, 1.0 = 100% exp
extern float exp_elevator;
extern float exp_rudder;
extern float exp_throttle;
extern float offset_aileron; // offset after exp mix, 0.0 = neutral, -0.5 .. 0.5
extern float offset_elevator;
extern float offset_rudder;
extern float offset_throttle;
extern int mouse_bind_x;
extern int mouse_bind_y;
extern int mouse_bind_l;
extern int mouse_bind_m;
extern int mouse_bind_r;
extern int mouse_bind_u;
extern int mouse_bind_d;
extern int joystick_bind_x[4]; // joystick axes
extern int joystick_bind_b[4]; // joystick buttons
extern int joystick_n;         // sdl joystick num
extern int radio_type;
extern int x_resolution;  // Screen resolution to use in X direction
extern int y_resolution;  // Screen resolution to use in Y direction
extern int color_depth;   // Color depth to use
extern int refresh_rate;  // Refresh rate to use
extern float dt;          // time interval of integration of EOMs
extern int initialization_time;    // Initial time when the program is started.
extern int current_time;           // Current time in ms, from sdl
extern  int time_after_last_integration;   //Time after the last integration of EOMs.
extern int last_time;     // time of last iteration is ms.
extern int paused;        // is the simulation paused?
extern int window_xsize;  // Size of window in x direction
extern int window_ysize;  // Size of window in y direction
extern int resuming;      //  Will the next iteration be resuming from a pause?
extern GLfloat field_of_view;  // Angular field of view in degrees.
extern int zoom_control;  // What input device controls the field of view
extern float shadow_matrix[4][4];  // Matrix used to calculate shadow on ground
extern float slope_shadow_matrix[4][4];  // Matrix used to calculate shadow on slope
extern float top_shadow_matrix[4][4];  // Matrix used to calculate shadow on slope
extern float cpu_speed;   // CPU speed in Mhz, for linux rctran interface
extern int shading;
extern int use_textures;
extern int use_mipmaps;
extern int stalling;

//Environment :
extern float wind_velocity;
extern float wind_direction;
extern thermal *thermals;
extern float thermal_density;         // How many thermals per sq mile
extern float thermal_strength_mean;   // Mean strength of thermal
extern float thermal_strength_sigma;  // 1 sigma variation in thermal strength
extern float thermal_radius_mean;     // Mean thermal radius
extern float thermal_radius_sigma;    // 1 sigma thermal radius variation
extern float thermal_lifetime_mean;   // Average lifetime of a thermal in minutes
extern float thermal_lifetime_sigma;  // 1 sigma variation in lifetime in minutes
extern int num_thermals;
extern int training_mode;             // Draw thermals in the sky?

//Sky :
extern char sky_texture_filename[];
extern char ground_texture_filename[];
extern char grass_texture_filename[];
extern char pine_texture_filename[];
extern char decid_texture_filename[];
extern char grass_side_texture_filename[];
extern char grass_top_texture_filename[];
extern char eastern_view_filename[];
extern char netrees_filename[];
extern char dirt_filename[];
extern char water_filename[];
extern char beachsand_filename[];
extern char scrub_filename[];
extern char scrubedge_filename[];
extern char south_filename[];
extern char hilledge_filename[];
extern char waves_filename[];
extern char sand_filename[];
extern char outhouse_filename[];
extern char freq_filename[];

//Airplane :
extern float aileron_input;       // Current aileron position
extern float elevator_input;      // Current elevator position
extern float rudder_input;        // Current rudder position
extern float throttle_input;      // Current throttle position
extern int dynamic_soaring;

#endif  // CRRC_MAIN_H
