/*****************************************************************************
Title: CRRCsim, the Charles River Radio Control Club Flight Simulator Project
Authors:
Jan Kansky:  Programming
Mark Drela:  Aerodynamics

Purpose:  The idea for CRRCsim is to take away any last excuse you might
have for not using a flight simulator to keep your thumbs certified for RC
flying.   It's free, it works, enjoy.  This is an open source project, so if
you don't like the way something works, help us fix it!

Thanks:
  Bruce Jackson for the LaRCsim framework.
  Flight Gear project for the sky sphere.

Contacts:
If you'd like to help with CRRCSIM, then send me an email!
    email                : kansky@ll.mit.edu
*****************************************************************************/

#include "crrc_main.h"

#ifndef IS_PAR
#define IS_PAR(PARAMETER_NAME) (!strcasecmp(parameter_name, PARAMETER_NAME))
#endif
#ifndef READ_PAR
#define READ_PAR() fscanf(config_file, "%s %[^\n]", text_value, trailing_comment)
#endif
#ifndef IS_PVAL
#define IS_PVAL(PARAMETER_VALUE) (!strcasecmp(text_value, PARAMETER_VALUE))
#endif
#ifndef READ_FLOAT
#define READ_FLOAT(VARIABLE) fscanf(config_file, "%f %[^\n]", &VARIABLE, trailing_comment)
#endif
#ifndef READ_INT
#define READ_INT(VARIABLE) fscanf(config_file, "%i %[^\n]", &VARIABLE, trailing_comment)
#endif
#ifndef READ_STRING
#define READ_STRING(STRING) fscanf(config_file, "%s %[^\n]", STRING, trailing_comment)
#endif

int read_config_file(char *filename)
{
	FILE *config_file;
	char parameter_name[256]; //  Name of parameter
	char trailing_comment[256];
	char text_value[256];

	config_file=fopen("crrcsim.cfg", "r");
	if (config_file == NULL) {
		fprintf(stderr, "Unable to open CRRCsim configuration file crrcsim.cfg\n");
		exit(1);
	}

	while (fscanf(config_file, "%s", parameter_name) != EOF) {
		if (IS_PAR("verbose")) {
			READ_PAR();
			if (IS_PVAL("TRUE")) {
				verbose = TRUE;
			}
		} else if (IS_PAR("location")) {
			READ_PAR();
			if (IS_PVAL("DAVIS")) {
				location = DAVIS;

			} else if (IS_PVAL("MEDFIELD")) {
				location = MEDFIELD;

			} else if (IS_PVAL("CAPE_COD")) {
				location = CAPE_COD;
			}
		} else if (IS_PAR("fullscreen")) {
			READ_PAR();
			if (IS_PVAL("TRUE")) {
				fullscreen = TRUE;

			} else if (IS_PVAL("FALSE")) {
				fullscreen = FALSE;
			}
		} else if (IS_PAR("training_mode")) {
			READ_PAR();
			if (IS_PVAL("TRUE")) {
				training_mode = TRUE;

			} else if (IS_PVAL("FALSE")) {
				training_mode = FALSE;
			}
		} else if (IS_PAR("radio_type")) {
			READ_PAR();
			if (IS_PVAL("FUTABA")) {
				radio_type = FUTABA;
				
			} else if (IS_PVAL("AIRTRONICS")) {
				radio_type = AIRTRONICS;

			} else if (IS_PVAL("HITEC")) {
				radio_type = HITEC;
				
			} else if (IS_PVAL("JR")) {
				radio_type = JR;
			}
		} else if (IS_PAR("input_method")) {
			READ_PAR();
			if (IS_PVAL("KEYBOARD")) {
				input_method = KEYBOARD;
				
			} else if (IS_PVAL("MOUSE")) {
				input_method = MOUSE;
				
			} else if (IS_PVAL("JOYSTICK")) {
				input_method = JOYSTICK;
				
			} else if (IS_PVAL("RCTRAN")) {
				input_method = RCTRAN;

			} else if (IS_PVAL("SERIAL")) {
				input_method = SERIAL;

			} else if (IS_PVAL("PARALLEL1")) {
				input_method = PARALLEL1;
				init_parallel_interface(1);
			} else if (IS_PVAL("PARALLEL2")) {
				input_method = PARALLEL2;
				init_parallel_interface(2);
			} else if (IS_PVAL("PARALLEL3")) {
				input_method = PARALLEL3;
				init_parallel_interface(3);
			}
		} else if (IS_PAR("scale_aileron")) {
			READ_FLOAT(scale_aileron);

		} else if (IS_PAR("scale_elevator")) {
			READ_FLOAT(scale_elevator);

		} else if (IS_PAR("scale_rudder")) {
			READ_FLOAT(scale_rudder);

		} else if (IS_PAR("scale_throttle")) {
			READ_FLOAT(scale_throttle);

		} else if (IS_PAR("trim_aileron")) {
			READ_FLOAT(trim_aileron);
			trim_aileron *= 0.01;
		} else if (IS_PAR("trim_elevator")) {
			READ_FLOAT(trim_elevator);
			trim_elevator *= 0.01;
		} else if (IS_PAR("trim_rudder")) {
			READ_FLOAT(trim_rudder);
			trim_rudder *= 0.01;
		} else if (IS_PAR("trim_throttle")) {
			READ_FLOAT(trim_throttle);
			trim_throttle *= 0.01;
		} else if (IS_PAR("nrate_aileron")) {
			READ_FLOAT(nrate_aileron);
			nrate_aileron *= 0.01;
		} else if (IS_PAR("nrate_elevator")) {
			READ_FLOAT(nrate_elevator);
			nrate_elevator *= 0.01;
		} else if (IS_PAR("nrate_rudder")) {
			READ_FLOAT(nrate_rudder);
			nrate_rudder *= 0.01;
		} else if (IS_PAR("nrate_throttle")) {
			READ_FLOAT(nrate_throttle);
			nrate_throttle *= 0.01;
		} else if (IS_PAR("exp_aileron")) {
			READ_FLOAT(exp_aileron);
			exp_aileron *= 0.01;
		} else if (IS_PAR("exp_elevator")) {
			READ_FLOAT(exp_elevator);
			exp_elevator *= 0.01;
		} else if (IS_PAR("exp_rudder")) {
			READ_FLOAT(exp_rudder);
			exp_rudder *= 0.01;
		} else if (IS_PAR("exp_throttle")) {
			READ_FLOAT(exp_throttle);
			exp_throttle *= 0.01;
		} else if (IS_PAR("offset_aileron")) {
			READ_FLOAT(offset_aileron);
			offset_aileron *= 0.01;
		} else if (IS_PAR("offset_elevator")) {
			READ_FLOAT(offset_elevator);
			offset_elevator *= 0.01;
		} else if (IS_PAR("offset_rudder")) {
			READ_FLOAT(offset_rudder);
			offset_rudder *= 0.01;
		} else if (IS_PAR("offset_throttle")) {
			READ_FLOAT(offset_throttle);
			offset_throttle *= 0.01;
		} else if (IS_PAR("mouse_x")) {
			READ_PAR();
			if (IS_PVAL("AILERON")) {
				mouse_bind_x = AILERON;

			} else if (IS_PVAL("ELEVATOR")) {
				mouse_bind_x = ELEVATOR;

			} else if (IS_PVAL("RUDDER")) {
				mouse_bind_x = RUDDER;

			} else if (IS_PVAL("THROTTLE")) {
				mouse_bind_x = THROTTLE;
			}
		} else if (IS_PAR("mouse_y")) {
			READ_PAR();
			if (IS_PVAL("AILERON")) {
				mouse_bind_y = AILERON;
				
			} else if (IS_PVAL("ELEVATOR")) {
				mouse_bind_y = ELEVATOR;

			} else if (IS_PVAL("RUDDER")) {
				mouse_bind_y = RUDDER;

			} else if (IS_PVAL("THROTTLE")) {
				mouse_bind_y = THROTTLE;
			}
		} else if (IS_PAR("joystick_x0")) {
			READ_PAR();
			if (IS_PVAL("AILERON")) {
				joystick_bind_x[0] = AILERON;
				
			} else if (IS_PVAL("ELEVATOR")) {
				joystick_bind_x[0] = ELEVATOR;

			} else if (IS_PVAL("RUDDER")) {
				joystick_bind_x[0] = RUDDER;

			} else if (IS_PVAL("THROTTLE")) {
				joystick_bind_x[0] = THROTTLE;
			}
		} else if (IS_PAR("joystick_x1")) {
			READ_PAR();
			if (IS_PVAL("AILERON")) {
				joystick_bind_x[1] = AILERON;

			} else if (IS_PVAL("ELEVATOR")) {
				joystick_bind_x[1] = ELEVATOR;

			} else if (IS_PVAL("RUDDER")) {
				joystick_bind_x[1] = RUDDER;

			} else if (IS_PVAL("THROTTLE")) {
				joystick_bind_x[1] = THROTTLE;
			}
		} else if (IS_PAR("joystick_x2")) {
			READ_PAR();
			if (IS_PVAL("AILERON")) {
				joystick_bind_x[2] = AILERON;

			} else if (IS_PVAL("ELEVATOR"))	{
				joystick_bind_x[2] = ELEVATOR;

			} else if (IS_PVAL("RUDDER")) {
				joystick_bind_x[2] = RUDDER;

			} else if (IS_PVAL("THROTTLE")) {
				joystick_bind_x[2] = THROTTLE;
			}
		} else if (IS_PAR("joystick_x3")) {
			READ_PAR();
			if (IS_PVAL("AILERON")) {
				joystick_bind_x[3] = AILERON;

			} else if (IS_PVAL("ELEVATOR"))	{
				joystick_bind_x[3] = ELEVATOR;

			} else if (IS_PVAL("RUDDER")) {
				joystick_bind_x[3] = RUDDER;

			} else if (IS_PVAL("THROTTLE"))	{
				joystick_bind_x[3] = THROTTLE;
			}
		} else if (IS_PAR("mouse_l")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				mouse_bind_l = RESUME;

			} else if (IS_PVAL("PAUSE")) {
				mouse_bind_l = PAUSE;

			} else if (IS_PVAL("RESET")) {
				mouse_bind_l = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				mouse_bind_l = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				mouse_bind_l = DECTHROTTLE;
			}
		} else if (IS_PAR("mouse_m")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				mouse_bind_m = RESUME;

			} else if (IS_PVAL("PAUSE")) {
				mouse_bind_m = PAUSE;

			} else if (IS_PVAL("RESET")) {
				mouse_bind_m = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				mouse_bind_m = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				mouse_bind_m = DECTHROTTLE;
			}
		} else if (IS_PAR("mouse_r")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				mouse_bind_r = RESUME;

			} else if (IS_PVAL("PAUSE")) {
				mouse_bind_r = PAUSE;

			} else if (IS_PVAL("RESET")) {
				mouse_bind_r = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				mouse_bind_r = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				mouse_bind_r = DECTHROTTLE;
			}
		} else if (IS_PAR("mouse_u")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				mouse_bind_u = RESUME;

			} else if (IS_PVAL("PAUSE")) {
				mouse_bind_u = PAUSE;

			} else if (IS_PVAL("RESET")) {
				mouse_bind_u = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				mouse_bind_u = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				mouse_bind_u = DECTHROTTLE;
			}
		} else if (IS_PAR("mouse_d")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				mouse_bind_d = RESUME;

			} else if (IS_PVAL("PAUSE")) {
				mouse_bind_d = PAUSE;

			} else if (IS_PVAL("RESET")) {
				mouse_bind_d = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				mouse_bind_d = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				mouse_bind_d = DECTHROTTLE;
			}
		} else if (IS_PAR("joystick_b0")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				joystick_bind_b[0] = RESUME;

			} else if (IS_PVAL("PAUSE")) {
				joystick_bind_b[0] = PAUSE;

			} else if (IS_PVAL("RESET")) {
				joystick_bind_b[0] = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				joystick_bind_b[0] = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				joystick_bind_b[0] = DECTHROTTLE;
			}
		} else if (IS_PAR("joystick_b1")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				joystick_bind_b[1] = RESUME;

			} else if (IS_PVAL("PAUSE")) {
				joystick_bind_b[1] = PAUSE;

			} else if (IS_PVAL("RESET")) {
				joystick_bind_b[1] = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				joystick_bind_b[1] = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				joystick_bind_b[1] = DECTHROTTLE;
			}
		} else if (IS_PAR("joystick_b2")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				joystick_bind_b[2] = RESUME;

			} else if (IS_PVAL("PAUSE")) {
				joystick_bind_b[2] = PAUSE;

			} else if (IS_PVAL("RESET")) {
				joystick_bind_b[2] = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				joystick_bind_b[2] = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				joystick_bind_b[2] = DECTHROTTLE;
			}
		} else if (IS_PAR("joystick_b3")) {
			READ_PAR();
			if (IS_PVAL("RESUME")) {
				joystick_bind_b[3] = RESUME;
				
			} else if (IS_PVAL("PAUSE")) {
				joystick_bind_b[3] = PAUSE;

			} else if (IS_PVAL("RESET")) {
				joystick_bind_b[3] = RESET;

			} else if (IS_PVAL("INCTHROTTLE")) {
				joystick_bind_b[3] = INCTHROTTLE;

			} else if (IS_PVAL("DECTHROTTLE")) {
				joystick_bind_b[3] = DECTHROTTLE;
			}
		} else if (IS_PAR("joystick_n")) {
			READ_INT(joystick_n);

		} else if (IS_PAR("x_resolution")) {
			READ_INT(x_resolution);

		} else if (IS_PAR("y_resolution")) {
			READ_INT(y_resolution);

		} else if (IS_PAR("color_depth")) {
			READ_INT(color_depth);

		} else if (IS_PAR("refresh_rate")) {
			READ_INT(refresh_rate);

		} else if (IS_PAR("dt")) {
			READ_FLOAT(dt);

		} else if (IS_PAR("field_of_view")) {
			READ_FLOAT(field_of_view);

		} else if (IS_PAR("zoom_control")) {
			READ_PAR();
			if (IS_PVAL("MOUSE")) {
				zoom_control = MOUSE;
				
			} else if (IS_PVAL("KEYBOARD")) {
				zoom_control = KEYBOARD;
			}
		} else if (IS_PAR("cpu_speed"))	{
			READ_FLOAT(cpu_speed);

		} else if (IS_PAR("wind_velocity")) {
			READ_FLOAT(wind_velocity);

		} else if (IS_PAR("wind_direction")) {
			READ_FLOAT(wind_direction);
			wind_direction = wind_direction + 270;
			if (wind_direction >= 360)
				wind_direction -= 360;

			if (location == CAPE_COD) {
				if ((wind_direction < 360) && (wind_direction > 180)) {
					if (wind_direction != 270)
						printf("Off normal winds not supported yet.\n");
					wind_direction = 270;
					printf("Sloping.\n");
					dynamic_soaring = FALSE;
				} else {
					if (wind_direction != 90)
						printf("Off normal winds not supported yet.\n");
					wind_direction = 90;
					printf("Dynamic soaring.\n");
					dynamic_soaring = TRUE;
				}
			}

		} else if (IS_PAR("thermal_strength_mean")) {
			READ_FLOAT(thermal_strength_mean);
			
		} else if (IS_PAR("thermal_strength_sigma")) {
			READ_FLOAT(thermal_strength_sigma);

		} else if (IS_PAR("thermal_radius_mean")) {
			READ_FLOAT(thermal_radius_mean);

		} else if (IS_PAR("thermal_radius_sigma")) {
			READ_FLOAT(thermal_radius_sigma);

		} else if (IS_PAR("thermal_lifetime_mean")) {
			READ_FLOAT(thermal_lifetime_mean);

		} else if (IS_PAR("thermal_lifetime_sigma")) {
			READ_FLOAT(thermal_lifetime_sigma);

		} else if (IS_PAR("thermal_density")) {
			READ_FLOAT(thermal_density);

		} else if (IS_PAR("sky_texture")) {
			READ_STRING(sky_texture_filename);

		} else if (IS_PAR("ground_texture")) {
			READ_STRING(ground_texture_filename);

		} else if (IS_PAR("grass_texture")) {
			READ_STRING(grass_texture_filename);

		} else if (IS_PAR("grasss_texture")) {
			READ_STRING(grass_side_texture_filename);

		} else if (IS_PAR("grasst_texture")) {
			READ_STRING(grass_top_texture_filename);

		} else if (IS_PAR("eastern_view")) {
			READ_STRING(eastern_view_filename);

		} else if (IS_PAR("netrees_tex")) {
			READ_STRING(netrees_filename);

		} else if (IS_PAR("dirt_texture")) {
			READ_STRING(dirt_filename);

		} else if (IS_PAR("water_texture")) {
			READ_STRING(water_filename);

		} else if (IS_PAR("beachsand_texture")) {
			READ_STRING(beachsand_filename);

		} else if (IS_PAR("scrub_texture")) {
			READ_STRING(scrub_filename);

		} else if (IS_PAR("scrubedge_texture"))	{
			READ_STRING(scrubedge_filename);

		} else if (IS_PAR("south_texture")) {
			READ_STRING(south_filename);

		} else if (IS_PAR("hilledge_texture")) {
			READ_STRING(hilledge_filename);

		} else if (IS_PAR("waves_texture")) {
			READ_STRING(waves_filename);

		} else if (IS_PAR("sand_texture")) {
			READ_STRING(sand_filename);

		} else if (IS_PAR("outhouse_tex")) {
			READ_STRING(outhouse_filename);

		} else if (IS_PAR("freq_texture")) {
			READ_STRING(freq_filename);

		} else if (IS_PAR("pine_texture")) {
			READ_STRING(pine_texture_filename);

		} else if (IS_PAR("decid_texture")) {
			READ_STRING(decid_texture_filename);

		} else if (IS_PAR("shading")) {
			READ_PAR();
			if (IS_PVAL("SMOOTH")) {
				shading = SMOOTH;
			} else if (IS_PVAL("FLAT")) {
				shading = FLAT;
			}
		} else if (IS_PAR("use_textures")) {
			READ_PAR();
			if (IS_PVAL("TRUE")) {
				use_textures = TRUE;
			} else if (IS_PVAL("FALSE")) {
				use_textures = FALSE;
			}
		} else if (IS_PAR("use_mipmaps")) {
			READ_PAR();
			if (IS_PVAL("TRUE")) {
				use_mipmaps = TRUE;
			} else if (IS_PVAL("FALSE")) {
				use_mipmaps = FALSE;
			}
		} else {
			fprintf(stderr, "unknown parameter: %s\n", parameter_name);
		}
	}
	printf("Read in CRRCsim configuration file sucessfully.\n");
	return 0;
}

